//---------------------------------------------------------------------------
// Klasse     : LCD_Text
// Datei      : LCD_Text.cpp
// Datum      : 11.06.2007
// Version    : 1.0
// Autor      : Karsten Donat
//---------------------------------------------------------------------------

// ToDo
// - Pin Definition fr Daten als Bitfeld
// - Datenbits in Schleifen bearbeiten
// - ifdef berall (4/8bit)

#include "LCD_Text.h"
#include <avr/pgmspace.h>

int LCD_Text_Create()
{
   // ToDo: Initialisierungsroutine

   LCD_Select_InstructionRegister();
   LCD_Select_Write();
   LCD_Start_Transfer();

   // Datenrichtung der Steuerleitungen als Ausgang setzen
/*   SetBit(LCD_RS_DDR,LCD_RS_PIN);
   SetBit(LCD_RW_DDR,LCD_RW_PIN);
   SetBit(LCD_E_DDR ,LCD_E_PIN );*/

   // kein Datentransfer
   LCD_Select_InstructionRegister();
   LCD_Select_Write();
   LCD_Stop_Transfer();
   LCD_Set_Byte(0);

   // HD44780U Rountine
//   #ifdef LCD_TYP=HD44780U_8Bit

      // Reset abwarten
      // Wait for more than 15 ms after VCC rises to 4.5 V
      // Wait for more than 40 ms after VCC rises to 2.7 V
      delay_ms_grob(40);


      // Software Initialisation
      // RS R/W DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
      //  0  0   0   0   1   1   *   *   *   *
      // Wait for more than 4.1 ms
      LCD_Write_Command(0b00110000,5);

      // RS R/W DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
      //  0  0   0   0   1   1   *   *   *   *
      // Wait for more than 100 s
      LCD_Write_Command(0b00110000,1);

      // RS R/W DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
      //  0  0   0   0   1   1   *   *   *   *
      LCD_Write_Command(0b00110000,1);
      LCD_Write_Command(0b00110000,1);

      // Setup
      // Number of display lines and character font      
      // RS R/W DB7 DB6 DB5 DB4 DB3 DB2 DB1 DB0
      //  0  0   0   0   1   DL  N   F   *   *
      // DL=0 4 Bit
      // DL=1 8 Bit
      // N Display lines    N=0 one line display
      //                    N=1 two lines display
      // F Character Font   F=0 5x8  (1|2 lines)
      //                    F=1 5x10 (1 line)
      LCD_Write_Command(0b00111000,1);

      // Display off
//00001000

      // Display on
      LCD_Write_Command(0b00001110,1);

      // Display clear
//00000001

      // Entry mode set
      //1 I/D S
      LCD_Entry_mode_set(_INCREMENT_,_LEFT_TO_RIGHT_);


      // Display lschen
      LCD_Clear_display();
      LCD_Display_On_Cursor_Off();


      // Startanzeige LCD-Treiber
      LCD_GotoXY(3,1);
      LCD_Write_String_Flash("donat-electronic  ATMega IDE 2007");
      LCD_GotoXY(4,2);
      LCD_Write_String_Flash("LCD-Treiber (Text) Version 1.10");
//      delay_ms_grob(5000);
      LCD_Clear_display();

//   #endif

   return 0;
}

int LCD_Text_Destroy()
{
   // ToDo: Aufrumen
   return 0;
}

// Clears entire display and sets DDRAM address 0 in address counter.
void LCD_Clear_display()
{
   LCD_Write_Command(0b00000001,1);
}

// Sets DDRAM address 0 in address counter.
// Also returns display from being shifted to original position.
// DDRAM contents remain unchanged.
void LCD_Return_home()
{
   LCD_Write_Command(0b00000010,2);
}

// Sets cursor move direction and specifies display shift.
// These operations are performed during data write and read.
void LCD_Entry_mode_set(unsigned char IncDec,unsigned char Direction)
{
   LCD_Write_Command(0b00000100+IncDec+Direction,1);
}

void LCD_Display_Off()
{
   LCD_Write_Command(0b00001000,1);
}

void LCD_Display_On_Cursor_Off()
{
   LCD_Write_Command(0b00001100,1);
}

void LCD_Display_On_Cursor_On()
{
   LCD_Write_Command(0b00001110,1);
}

void LCD_Display_On_Cursor_Blinking()
{
   LCD_Write_Command(0b00001101,1);
}

void LCD_Display_On_Cursor_Blinking_Both()
{
   LCD_Write_Command(0b00001111,1);
}

//Moves cursor and shifts display without changing DDRAM contents.
//Cursor or display shift
//0001 S/C R/L 00

void LCD_GotoXY(int x, int y)
{
   LCD_Write_Command(0b01000000+--x+(--y<<7),1);
}



void LCD_Write_String(const char* pText)
{
   while(pText[0]!=0)
   {
      LCD_Write_Char(pText[0],0);
      pText++;
   }
}

void LCD_Write_String_Flash_p(const char *progmem_s)
{
   register char c;

   while ( (c = pgm_read_byte(progmem_s++)) ) 
      LCD_Write_Char(c,0);
}

void LCD_Write_Byte(Byte Zahl)
{
      char Buffer[4];
      IntToStr(Zahl,Buffer);LCD_Write_String(Buffer);
}
void LCD_Write_Word(Word Zahl)
{
      char Buffer[6];
      IntToStr(Zahl,Buffer);LCD_Write_String(Buffer);
}

void LCD_Write_Command(unsigned char Command, double Pause)
{
   LCD_Select_InstructionRegister();
   LCD_Select_Write();
   LCD_Set_Byte(Command);
   LCD_Start_Transfer();

   // Wait
   delay_ms_grob(Pause);

   LCD_Stop_Transfer();
}

unsigned char LCD_Read_Command(unsigned char Command, double Pause)
{
   return 0;
}

void LCD_Write_Char(unsigned char Daten, double Pause)
{
   // Zeichencodes nach japanischem Zeichendatz konvertieren
   #if LCD_Charset==A00_Japan
           // @...[
      if ((Daten>=64)and(Daten<=91)) {Daten=Daten+(128-64);}
      // 
//      else if (Daten=92) {Daten=U1;}
       // ]
      else if ((Daten>=93)and(Daten<=125)) {Daten=Daten+(157-93);}
      // 
      else if (Daten==129) {Daten=245;}
      // 
//      else if (Daten==132) {Daten=225;} // eigentlicher Windows Code fr 
      else if (Daten==228) {Daten=225;} //  durch WinAVR anders codiert
      // 
      else if (Daten==148) {Daten=239;}
      // 
      else if (Daten==190) {Daten=156;}
      // 
      else if (Daten==158) {Daten=235;}
      // 
      else if (Daten==219) {Daten=255;}
      // 
      else if (Daten==225) {Daten=226;}
      // 
      else if (Daten==230) {Daten=228;}
      // 
      else if (Daten==246) {Daten=253;}
      // 
      else if (Daten==248) {Daten=223;}
   #endif


   LCD_Select_DataRegister();
   LCD_Select_Write();
   LCD_Set_Byte(Daten);
   LCD_Start_Transfer();

   // Wait
   delay_ms_grob(Pause);

   LCD_Stop_Transfer();
}

unsigned char LCD_Read_Char(unsigned char Command, double Pause)
{
   return 0;
}



void LCD_Select_InstructionRegister()
{
   SetBit(LCD_RS_DDR,LCD_RS_PIN);
   ClearBit(LCD_RS_PORT,LCD_RS_PIN);
}
void LCD_Select_DataRegister()
{
   SetBit(LCD_RS_DDR,LCD_RS_PIN);
   SetBit(LCD_RS_PORT,LCD_RS_PIN);
}

void LCD_Select_Read()
{
   // Daten lesen vorbereiten
   SetBit(LCD_RW_DDR,LCD_RW_PIN);
   SetBit(LCD_RW_PORT,LCD_RW_PIN);

   // Datenpins auf Eingang setzen
   ClearBit(LCD_D0_DDR,LCD_D0_PIN);
   ClearBit(LCD_D1_DDR,LCD_D1_PIN);
   ClearBit(LCD_D2_DDR,LCD_D2_PIN);
   ClearBit(LCD_D3_DDR,LCD_D3_PIN);
   ClearBit(LCD_D4_DDR,LCD_D4_PIN);
   ClearBit(LCD_D5_DDR,LCD_D5_PIN);
   ClearBit(LCD_D6_DDR,LCD_D6_PIN);
   ClearBit(LCD_D7_DDR,LCD_D7_PIN);
}
void LCD_Select_Write()
{
   // Daten lesen vorbereiten
   SetBit(LCD_RW_DDR,LCD_RW_PIN);
   ClearBit(LCD_RW_PORT,LCD_RW_PIN);

   // Datenpins auf Ausgang setzen
   SetBit(LCD_D0_DDR,LCD_D0_PIN);
   SetBit(LCD_D1_DDR,LCD_D1_PIN);
   SetBit(LCD_D2_DDR,LCD_D2_PIN);
   SetBit(LCD_D3_DDR,LCD_D3_PIN);
   SetBit(LCD_D4_DDR,LCD_D4_PIN);
   SetBit(LCD_D5_DDR,LCD_D5_PIN);
   SetBit(LCD_D6_DDR,LCD_D6_PIN);
   SetBit(LCD_D7_DDR,LCD_D7_PIN);
}

void LCD_Start_Transfer()
{
   SetBit(LCD_E_DDR ,LCD_E_PIN );
   SetBit(LCD_E_PORT,LCD_E_PIN);
}
void LCD_Stop_Transfer()
{
   SetBit(LCD_E_DDR ,LCD_E_PIN );
   ClearBit(LCD_E_PORT,LCD_E_PIN);
}

void LCD_Set_Byte(unsigned char Daten)   // schreibt das Daten-Byte in die entsprechenden Register
{
   LCD_D0_PORT = (LCD_D0_PORT & ~(1<<LCD_D0_PIN)) + (((Daten & (1<<0))>>0) <<LCD_D0_PIN);
   LCD_D1_PORT = (LCD_D1_PORT & ~(1<<LCD_D1_PIN)) + (((Daten & (1<<1))>>1) <<LCD_D1_PIN);
   LCD_D2_PORT = (LCD_D2_PORT & ~(1<<LCD_D2_PIN)) + (((Daten & (1<<2))>>2) <<LCD_D2_PIN);
   LCD_D3_PORT = (LCD_D3_PORT & ~(1<<LCD_D3_PIN)) + (((Daten & (1<<3))>>3) <<LCD_D3_PIN);
   LCD_D4_PORT = (LCD_D4_PORT & ~(1<<LCD_D4_PIN)) + (((Daten & (1<<4))>>4) <<LCD_D4_PIN);
   LCD_D5_PORT = (LCD_D5_PORT & ~(1<<LCD_D5_PIN)) + (((Daten & (1<<5))>>5) <<LCD_D5_PIN);
   LCD_D6_PORT = (LCD_D6_PORT & ~(1<<LCD_D6_PIN)) + (((Daten & (1<<6))>>6) <<LCD_D6_PIN);
   LCD_D7_PORT = (LCD_D7_PORT & ~(1<<LCD_D7_PIN)) + (((Daten & (1<<7))>>7) <<LCD_D7_PIN);
}
unsigned char LCD_Get_Byte()            // liest das Daten-Byte aus
{
   return 0;
}
