//---------------------------------------------------------------------------
// Klasse     : TCP
// Datei      : TCP.h
// Datum      : 11.07.2007
// Version    : 1.0
// Autor      : Karsten Donat
//---------------------------------------------------------------------------

#ifndef _TCP_H_
#define _TCP_H_

#include "Standard.h"

#define TCP_Header_Laenge 20 // evtl. mehr mit Options
#define TCP_Begin 34 // ab hier TCP Packet (inkl. Header)

struct TCP_Header
{
   Byte Source_Port_Hi;
   Byte Source_Port_Lo;
   Byte Destination_Port_Hi;
   Byte Destination_Port_Lo;
   Byte Sequence_Number_B3;
   Byte Sequence_Number_B2;
   Byte Sequence_Number_B1;
   Byte Sequence_Number_B0;
   Byte Acknowlegement_Number_B3;
   Byte Acknowlegement_Number_B2;
   Byte Acknowlegement_Number_B1;
   Byte Acknowlegement_Number_B0;

   struct
   {
      Byte Reserved:4;
      Byte Val     :4;
   } DataOffset;      // gibt die Anzahl der 32-Bit Wrter des Headers an (5 ohne Options, sonst mehr), danach beginnen die Daten
   
   // Flags -> umgekehrte Bitfolge!
   union
   {
      struct
      {
         Byte FIN:1; // Ende
         Byte SYN:1; // Synchronize Sequenz Numbers
         Byte RST:1; // Reset
         Byte PSH:1; // Push
         Byte ACK:1; // Acknowledgement
         Byte URG:1; // Urgent
         Byte ECN:1; // ECN-Echo
         Byte CWR:1; // Conngestion Window Reduced
      } bits;
      Byte byte;
   } Flags;

   Byte Window_size_Hi; // ab sofort maximal erlaubte Fenstergre
   Byte Window_size_Lo; // ab sofort maximal erlaubte Fenstergre
   Byte Checksum_Hi;
   Byte Checksum_Lo;
   Byte Urgent_Pointer_Hi; // nur vorhanden wenn URG gesetzt, sonst ggf. Options
   Byte Urgent_Pointer_Lo; // nur vorhanden wenn URG gesetzt, sonst ggf. Options
   //DWord Options_Padding; // Optionen inkl. Padding auf 32 Bit
};

// TCP Options as defined by RFC
#define TCP_OPTIONS_END_OF_LIST     (0x00u)
#define TCP_OPTIONS_NO_OP           (0x01u)
#define TCP_OPTIONS_MAX_SEG_SIZE    (0x02u)
typedef struct _TCP_OPTIONS
{
   Byte Kind;
   Byte Length;
   Word MaxSegSize;
} TCP_OPTIONS;

#define SwapPseudoTCPHeader(h)  (h.TCPLength = swaps(h.TCPLength))

#define IP_ADDR DWord
// IP pseudo header as defined by RFC 793
typedef struct
{
//   IP_ADDR SourceAddress;
//   IP_ADDR DestAddress;
   Byte SourceAddress[4];
   Byte DestAddress[4];
   Byte Zero;
   Byte Protocol;
   Word TCPLength;
} TCP_Pseudo_Header;


#define TCP_Socket_Anzahl 5 // maximale Anzahl mglicher Sockets

struct TCP_Socket
{
//   Byte Client_IP[4];
   DWord Client_IP;
   Word Client_Port; // Port der Anwendung auf den anfragenden Client
   Word Server_Port; // Port der Anwendung auf dem ATMega
   DWord Last_Tick;  // letzter Tickwert um Zeitdauer seit letztem Austausch zu berechnen
   Byte  Phase;      /* TCP Vermittlungsphase
                        0 - geschlossen
                        1 - Verbindung wurde angefragt
                        2 - Verbindung wurde akzeptiert
                        3 - Verbindung besteht */
};

#define TCP_Socket_geschlossen       0                                ==>  1
#define TCP_Socket_SYN_Anfrage       1 // -> [SYN]                    ==>  2,    0 (TimeOut)
#define TCP_Socket_SYN_Akzeptiert    2 // <- [SYN,ACK]                ==>  3,    0 (TimeOut)
#define TCP_Socket_SYN_Bestaetigung  3 // -> [ACK]                    ==>  4,    0 (TimeOut)
#define TCP_Socket_GET_Anfrage       4 // -> [ACK PSH] GET /test.html ==>  5,    0 (TimeOut)
#define TCP_Socket_GET_Bestaetigung  5 // <- [ACK]                    ==>  6,    0 (TimeOut)
#define TCP_Socket_HTML_Senden       6 // <- [ACK PSH] HTML           ==>  4, 7, 0 (TimeOut)
#define TCP_Socket_HTML_Bestaetigen  7 // -> [ACK]                    ==>  8,    0 (TimeOut)
#define TCP_Socket_FIN_Beenden       8 // <- [ACK FIN]                ==>  9,    0 (TimeOut)
#define TCP_Socket_FIN_Empfangen     9 // -> [ACK]                    ==> 10,    0 (TimeOut)
#define TCP_Socket_FIN_Akzeptiert   10 // -> [FIN ACK]                ==> 11,    0 (TimeOut)
#define TCP_Socket_FIN_Ende         11 // <- [ACK]                    ==>  0
//#define TCP_Socket_ 12 // -> [RST ?]
//#define TCP_Socket_ 13 // <- [RST ?]


int TCP_Create();
int TCP_Destroy();

void TCP_Verarbeite_Packet(Word len, Byte *buff);


#endif

